package codex

import (
  "fmt"
  "strconv"
)

type CodexLVar struct {
  id           string
  PacketLength int
  length        int
}

func (c *CodexLVar) init(cfg map[string]interface{}) {
  if v, ok := cfg["id"].(string); ok {
    c.id = v
  } else {
    panic(fmt.Sprintf("Invalid CodexLVar.ID %+v", cfg))
  }
  if v, ok := cfg["length"].(int); ok {
    c.length = v
  } else if cfg["length"] == nil {
    c.length = 0
  } else {
    panic(fmt.Sprintf("Invalid CodexLVar.Length %+v", cfg))
  }
}

func (c *CodexLVar) ID() string {
  return c.id
}

func (c *CodexLVar) decode(data []byte, pos int) (string, int, error) {
  ppos := pos + c.PacketLength
  if len(data) < ppos {
    return "", pos, fmt.Errorf("incomplete packet field %s %d < %d", c.id, len(data), ppos)
  }
  plen, err := strconv.Atoi(string(data[pos:ppos]))
  if err != nil {
    return "", pos, fmt.Errorf("Invalid lvar %s length [%s] %s", c.id, string(data[pos:ppos]), err)
  }
  ppos1 := ppos
  ppos = ppos + plen
  if len(data) < ppos {
    return "", pos, fmt.Errorf("incomplete packet field %s %d < %d", c.id, len(data), ppos)
  }
  return string(data[ppos1:ppos]), ppos, nil
}

func (c *CodexLVar) Decode(data []byte, pos int, value interface{}) (int, error) {
  var err error
  if vm, ok := value.(map[string]interface{}); ok {
    vm[c.id], pos, err = c.decode(data, pos)
    if err != nil {
      return pos, err
    }
    return pos, nil
  }
  panic(fmt.Sprintf("Invalid value type %+v", value))
}
